//---------------------------------------------------------------------------

#include <vcl.h>
#pragma hdrstop

#include "Unit1.h"
#include "direct.h"
#include "stdio.h"

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"
TMainForm *MainForm;
//---------------------------------------------------------------------------
__fastcall TMainForm::TMainForm(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------
//The progress's call-back function.
//To set conversion progress.
void GGetProgress(long nOther,long iProgress)
{
    TMainForm *pThis = (TMainForm *)nOther;
    pThis->m_ProgressBar->Position = iProgress;
}

//Demo1 - Convert all pdf pages.
void __fastcall TMainForm::btnDemo1Click(TObject *Sender)
{
    char sPdfFile[] = "sample.pdf";
    long nHandle;
    long iStartTick,iEndTick;

    //Loading dynamic link library---pdf2image.dll.
    if( LoadDLL() == FALSE ){
        ShowMessage("Loading dynamic link library---pdf2image.dll failure!");
        return;
    }

    iStartTick = GetTickCount();

    //Open pdf file and creating a converter.
    nHandle = _apOpen(sPdfFile,NULL,NULL);
    switch( nHandle ){
    case ERR_OVER_MAXTHREADS: {
        ShowMessage("Over the limit amount of threads,please waiting!");
        FreeDLL(); return;
    }
    case ERR_FILE_UNEXIST: {
        ShowMessage("Source pdf file unexist!");
        FreeDLL(); return;
    }
    case ERR_FILE_DAMAGED: {
        ShowMessage("Source pdf file damaged!");
        FreeDLL(); return;
    }
    case ERR_FILE_RESTRICTED: {
        ShowMessage("Source pdf file restricted!");
        FreeDLL(); return;
    }}

    //Setting the converter's proerties.
    //For more,please refer to "apconvert.h".
    _apSetProperty(nHandle,AP_PROP_OUTDIR,"./demo1",0,0);//output directory
    _apSetProperty(nHandle,AP_PROP_PREFIX,"myimg",0,0);//prefix of image file name
    _apSetProperty(nHandle,AP_PROP_IMAGETYPE,NULL,AP_IMAGE_BMP,0);//image type
    _apSetProperty(nHandle,AP_PROP_XDPI,NULL,120,0);//horizontal dpi
    _apSetProperty(nHandle,AP_PROP_YDPI,NULL,120,0);//vertical dpi
    
    //Setting the progress's call-back function.
    //The last parameter will be used to call-back function as a parameter value.
    _apSetProperty(nHandle,AP_PROP_PROGRESSFUNC,NULL,(long)GGetProgress,(long)this);

    //To convert the pdf file.
    _apConvert(nHandle);

    //Close pdf file and destory the converter.
    _apClose(nHandle);

    //To count conversion's time(ms).
    iEndTick = GetTickCount();
    ShowMessage("Conversion successful,it passed "+IntToStr(iEndTick-iStartTick)+"ms!");

    FreeDLL();
}

//---------------------------------------------------------------------------
//Demo2 - Convert appointed pdf pages.
void __fastcall TMainForm::btnDemo2Click(TObject *Sender)
{
    char sPdfFile[] = "sample.pdf";
    long m_Pages[] = {7,7,19,15,2};
    long nHandle;
    long iStartTick,iEndTick;

    //Loading dynamic link library---pdf2image.dll.
    if( LoadDLL() == FALSE ){
        ShowMessage("Loading dynamic link library---pdf2image.dll failure!");
        return;
    }

    iStartTick = GetTickCount();

    //Open pdf file and creating a converter.
    nHandle = _apOpen(sPdfFile,NULL,NULL);
    switch( nHandle ){
    case ERR_OVER_MAXTHREADS: {
        ShowMessage("Over the limit amount of threads,please waiting!");
        FreeDLL(); return;
    }
    case ERR_FILE_UNEXIST: {
        ShowMessage("Source pdf file unexist!");
        FreeDLL(); return;
    }
    case ERR_FILE_DAMAGED: {
        ShowMessage("Source pdf file damaged!");
        FreeDLL(); return;
    }
    case ERR_FILE_RESTRICTED: {
        ShowMessage("Source pdf file restricted!");
        FreeDLL(); return;
    }}

    //Setting the converter's propertys.
    //For more,please refer to "apconvert.h".
    _apSetProperty(nHandle,AP_PROP_OUTDIR,"./demo2",0,0);//output directory
    _apSetProperty(nHandle,AP_PROP_IMAGETYPE,NULL,AP_IMAGE_TIFF,0);//image type
    _apSetProperty(nHandle,AP_PROP_BITCOUNT,NULL,24,0);//color bits
    _apSetProperty(nHandle,AP_PROP_XDPI,NULL,150,0);//horizontal dpi
    _apSetProperty(nHandle,AP_PROP_YDPI,NULL,150,0);//vertical dpi
    _apSetProperty(nHandle,AP_PROP_COMPRESSION,NULL,AP_TIFF_COMPRESSION_LZW,0);//tiff compression mode
    _apSetProperty(nHandle,AP_PROP_MULTIPAGES,NULL,TRUE,0);//multipages tiff.

    //To convert the appointed pages of the pdf file.
    int iPageCount = sizeof(m_Pages)/sizeof(long);
    for(int iPos = 0; iPos < iPageCount; iPos++){
        _apConvertPage(nHandle,m_Pages[iPos]);

        //Setting conversion's progress.
        m_ProgressBar->Position = (long)((iPos+1)*1.0/iPageCount*100);
    }

    //Close pdf file and destory the converter.
    _apClose(nHandle);

    //To count conversion's time(ms).
    iEndTick = GetTickCount();
    ShowMessage("Conversion successful,it passed "+IntToStr(iEndTick-iStartTick)+"ms!");

    FreeDLL();
}
//---------------------------------------------------------------------------
//demo3 - Convert appointed pdf pages to memory stream.
void __fastcall TMainForm::btnDemo3Click(TObject *Sender)
{
    char sPdfFile[] = "sample.pdf";
    long m_Pages[] = {1,4,7,11,20,19,13,5,6,14,8};
    long nHandle;
    long iStartTick,iEndTick;

    //Loading dynamic link library---pdf2image.dll.
    if( LoadDLL() == FALSE ){
        ShowMessage("Loading dynamic link library---pdf2image.dll failure!");
        return;
    }

    iStartTick = GetTickCount();

    //Open pdf file and creating a converter.
    nHandle = _apOpen(sPdfFile,NULL,NULL);
    switch( nHandle ){
    case ERR_OVER_MAXTHREADS: {
        ShowMessage("Over the limit amount of threads,please waiting!");
        FreeDLL(); return;
    }
    case ERR_FILE_UNEXIST: {
        ShowMessage("Source pdf file unexist!");
        FreeDLL(); return;
    }
    case ERR_FILE_DAMAGED: {
        ShowMessage("Source pdf file damaged!");
        FreeDLL(); return;
    }
    case ERR_FILE_RESTRICTED: {
        ShowMessage("Source pdf file restricted!");
        FreeDLL(); return;
    }}

    //Setting the converter's propertys.
    //For more,please refer to "apconvert.h".
    _apSetProperty(nHandle,AP_PROP_IMAGETYPE,NULL,AP_IMAGE_WMF,0);
    _apSetProperty(nHandle,AP_PROP_XDIMENSIONS,NULL,1122,0);
    _apSetProperty(nHandle,AP_PROP_YDIMENSIONS,NULL,1452,0);
    
    //To convert the appointed pages of the pdf file.
    BYTE *pImageStream = NULL;
    long nSize;

    int iPageCount = sizeof(m_Pages)/sizeof(long);
    for(int iPos = 0; iPos < iPageCount; iPos++){
        //You can save image stream to database, to file, or to others, etc.
        //The "pImageStream" is a pointer which point to memory image stream.
        _apConvertPageToStream(nHandle,m_Pages[iPos],pImageStream,nSize);

        //Here,the image stream is saved to file.
        if( pImageStream != NULL && nSize > 0 ) {
            char sImgFile[256];
            _mkdir("./demo3");
            sprintf(sImgFile,"./demo3/myimg_%02d.wmf",m_Pages[iPos]);

            FILE *pFile = fopen(sImgFile,"wb");
            if( pFile != NULL ){
                fwrite(pImageStream,1,nSize,pFile);
                fclose(pFile);
            }
        }

        //Setting conversion's progress.
        m_ProgressBar->Position = (long)((iPos+1)*1.0/iPageCount*100);
    }

    //Close pdf file and destory the converter.
    _apClose(nHandle);

    //To count conversion's time(ms).
    iEndTick = GetTickCount();
    ShowMessage("Conversion successful,it passed "+IntToStr(iEndTick-iStartTick)+"ms!");

    FreeDLL();
}
//---------------------------------------------------------------------------

//Load dynamic link library---pdf2image.dll!
BOOL TMainForm::LoadDLL(){
	char sDllFile[]="pdf2image.dll";

	m_hDll = LoadLibrary(sDllFile);
	if( m_hDll == NULL ) return FALSE;

	_apOpen = (_FapOpen *)GetProcAddress(m_hDll, "apOpen");
	if( _apOpen == NULL ) goto ERR_FLAG;

	_apClose = (_FapClose *)GetProcAddress(m_hDll, "apClose");
	if( _apClose == NULL ) goto ERR_FLAG;

	_apConvert = (_FapConvert *)GetProcAddress(m_hDll, "apConvert");
	if( _apConvert == NULL ) goto ERR_FLAG;

	_apConvertPage = (_FapConvertPage *)GetProcAddress(m_hDll, "apConvertPage");
	if( _apConvertPage == NULL ) goto ERR_FLAG;

	_apConvertPageToStream = (_FapConvertPageToStream *)GetProcAddress(m_hDll, "apConvertPageToStream");
	if( _apConvertPageToStream == NULL ) goto ERR_FLAG;

	_apSetProperty = (_FapSetProperty *)GetProcAddress(m_hDll, "apSetProperty");
	if( _apSetProperty == NULL ) goto ERR_FLAG;

	_apGetProperty = (_FapGetProperty *)GetProcAddress(m_hDll, "apGetProperty");
	if( _apGetProperty == NULL ) goto ERR_FLAG;

	return TRUE;
ERR_FLAG:
	FreeDLL();
	return FALSE;
}

//Free dynamic link library---pdf2image.dll!
BOOL TMainForm::FreeDLL(){
	if( m_hDll ){
        FreeLibrary(m_hDll);
        m_hDll = NULL;
		_apOpen = NULL;
		_apClose = NULL;
		_apConvert = NULL;
		_apConvertPage = NULL;
		_apConvertPageToStream = NULL;
		_apSetProperty = NULL;
		_apGetProperty = NULL;
	}
	return TRUE;
}

